@@ -1,4 +1,4 @@
-# Copyright (C) 2021,2022,2023 Kian-Meng Ang
+# Copyright (C) 2021,2022,2023,2024 Kian-Meng Ang
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -21,11 +21,34 @@ from typing import Union
 import txt2ebook.models.book
 from txt2ebook.formats.epub import TEMPLATES as EPUB_TEMPLATES
 from txt2ebook.formats.epub import EpubWriter
+from txt2ebook.formats.gmi import GmiWriter
+from txt2ebook.formats.md import MdWriter
+from txt2ebook.formats.pdf import PdfWriter
 from txt2ebook.formats.txt import TxtWriter
+from txt2ebook.formats.typ import TypWriter
 from txt2ebook.helpers import load_class, to_classname
 from txt2ebook.models import Book
 
-EBOOK_FORMATS = ["epub", "txt"]
+EBOOK_FORMATS = ["epub", "gmi", "md", "pdf", "txt", "typ"]
+PAGE_SIZES = [
+    "a0",
+    "a1",
+    "a2",
+    "a3",
+    "a4",
+    "a5",
+    "a6",
+    "b0",
+    "b1",
+    "b2",
+    "b3",
+    "b4",
+    "b5",
+    "b6",
+    "elevenseventeen",
+    "legal",
+    "letter",
+]
 
 
 def create_format(
@@ -55,5 +78,10 @@ __all__ = [
     "EBOOK_FORMATS",
     "EPUB_TEMPLATES",
     "EpubWriter",
+    "GmiWriter",
+    "MdWriter",
+    "PAGE_SIZES",
+    "PdfWriter",
     "TxtWriter",
+    "TypWriter",
 ]
