@@ -1,4 +1,4 @@
-# Copyright (C) 2021,2022,2023 Kian-Meng Ang
+# Copyright (C) 2021,2022,2023,2024 Kian-Meng Ang
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -15,18 +15,17 @@
 
 """Convert source text file into EPUB format."""
 
-import gettext
-import io
 import logging
 import uuid
 from importlib.resources import contents, read_text
 from pathlib import Path
-from typing import Optional, Union
+from typing import Optional
 
 from ebooklib import epub
 
+from txt2ebook.formats.base import BaseWriter
 from txt2ebook.formats.templates import epub as template_epub
-from txt2ebook.models import Book, Chapter, Volume
+from txt2ebook.models import Chapter, Volume
 
 logger = logging.getLogger(__name__)
 
@@ -37,46 +36,9 @@ TEMPLATES = [
 ]
 
 
-class EpubWriter:
+class EpubWriter(BaseWriter):
     """Module for writing ebook in epub format."""
 
-    def __init__(self, book: Book, config: dict) -> None:
-        """Create a EpubWriter module.
-
-        Args:
-            book(Book): The book model which contains list of chapters and
-            other settings.
-            opts(dict): The configs from the command-line.
-
-        Returns:
-            None
-        """
-        self.book = book
-        self.config = config
-        self._ = self._load_translation()
-
-    def __getattr__(self, key: str) -> Optional[Union[str, bool]]:
-        """Get a value of the config based on key name.
-
-        Args:
-            key(str): The key name of the config.
-
-        Returns:
-            Any: The value of a key, if found. Otherwise raise AttributeError
-            exception.
-        """
-        if hasattr(self.config, key):
-            return getattr(self.config, key)
-
-        raise AttributeError(f"invalid config key: '{key}'!")
-
-    def _load_translation(self):
-        localedir = Path(Path(__file__).parent.parent, "locales")
-        translation = gettext.translation(
-            "txt2ebook", localedir=localedir, languages=[self.config.language]
-        )
-        return translation.gettext
-
     def write(self) -> None:
         """Generate the epub file."""
         book = epub.EpubBook()
@@ -115,7 +77,7 @@ class EpubWriter:
                     book.spine.append(html_chapter)
                     html_chapters.append(html_chapter)
 
-                if self.volume_page:
+                if self.config.volume_page:
                     logger.debug("Create separate volume page: %s", section)
                     book.toc.append((html_volume, html_chapters))
                 else:
@@ -129,17 +91,17 @@ class EpubWriter:
                 book.spine.append(html_chapter)
                 book.toc.append(html_chapter)
 
-        output_filename = self._gen_output_filename()
+        output_filename = self._output_filename(".epub")
         output_filename.parent.mkdir(parents=True, exist_ok=True)
         epub.write_epub(output_filename, book, {})
-        logger.info("Generate EPUB file: %s", output_filename)
+        logger.info("Generate EPUB file: %s", output_filename.resolve())
 
     def _build_nav(self, book: epub.EpubBook) -> None:
         book.add_item(epub.EpubNcx())
 
         try:
-            logger.info("Using EPUB template: %s", self.epub_template)
-            css = read_text(template_epub, f"{self.epub_template}.css")
+            logger.info("Using EPUB template: %s", self.config.epub_template)
+            css = read_text(template_epub, f"{self.config.epub_template}.css")
 
             book_css = epub.EpubItem(
                 uid="style_nav",
@@ -157,7 +119,9 @@ class EpubWriter:
             book.spine.append("nav")
 
         except FileNotFoundError as error:
-            logger.error("Unknown EPUB template name: %s", self.epub_template)
+            logger.error(
+                "Unknown EPUB template name: %s", self.config.epub_template
+            )
             raise SystemExit() from error
 
     def _gen_id(self) -> str:
@@ -169,23 +133,6 @@ class EpubWriter:
         """
         return str(uuid.uuid5(uuid.NAMESPACE_DNS, self.book.title))
 
-    def _gen_output_filename(self) -> Path:
-        """Generate the output EPUB filename."""
-        filename = "default"
-
-        if self.output_file:
-            filename = str(self.output_file)
-        elif isinstance(
-            self.input_file, (io.TextIOWrapper, io.BufferedReader)
-        ):
-            if self.input_file.name != "<stdin>":
-                filename = self.input_file.name
-        elif self.book.title:
-            filename = self.book.title
-
-        file = Path(filename)
-        return Path(file.parent, file.stem).with_suffix(".epub")
-
     def _build_cover(self) -> epub.EpubHtml:
         """Generate the cover image."""
         html = """
