@@ -1,4 +1,4 @@
-# Copyright (C) 2021,2022,2023 Kian-Meng Ang
+# Copyright (C) 2021,2022,2023,2024 Kian-Meng Ang
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -31,7 +31,7 @@ RE_CHAPTERS = [
     "声明.*",
     "引子.*",
     "楔子.*",
-    "序[章幕曲]?",
+    "[自]?序[章幕曲]?",
     "序.*",
     "前言.*",
     "(?:内容)?简介",
@@ -44,10 +44,19 @@ RE_CHAPTERS = [
     "尾声.*",
     "人物[介绍简介设定].*",
     "作者[介绍自介].*",
+    "正文",
+    "后记",
+    "人物谱",
 ]
 
-DEFAULT_RE_TITLE = r"书名：(.*)"
 DEFAULT_RE_AUTHOR = r"作者：(.*)"
-DEFAULT_RE_TAG = r"标签：(.*)"
-DEFAULT_RE_VOLUME = "(" + "|".join(RE_VOLUMES) + ")"
 DEFAULT_RE_CHAPTER = "(" + "|".join(RE_CHAPTERS) + ")"
+DEFAULT_RE_TAG = r"标签：(.*)"
+DEFAULT_RE_TITLE = r"书名：(.*)"
+DEFAULT_RE_TRANSLATOR = r"翻译：(.*)"
+DEFAULT_RE_VOLUME = "(" + "|".join(RE_VOLUMES) + ")"
+
+DEFAULT_PDF_FONT_FILE = "uming.ttc"
+DEFAULT_PDF_FONT_NAME = "AR PL UMing CN"
+DEFAULT_PDF_FONT_SIZE = 12
+DEFAULT_PDF_PAGE_SIZE = "A5"
