@@ -1,4 +1,4 @@
-# Copyright (C) 2021,2022,2023 Kian-Meng Ang
+# Copyright (C) 2021,2022,2023,2024 Kian-Meng Ang
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -18,7 +18,9 @@
 from txt2ebook.languages.zh_cn import (
     DEFAULT_RE_AUTHOR,
     DEFAULT_RE_CHAPTER,
+    DEFAULT_RE_TAG,
     DEFAULT_RE_TITLE,
+    DEFAULT_RE_TRANSLATOR,
     DEFAULT_RE_VOLUME,
     FULLWIDTH_NUMS,
     HALFWIDTH_NUMS,
@@ -28,10 +30,17 @@ from txt2ebook.languages.zh_cn import (
     SPACE,
 )
 
+DEFAULT_PDF_FONT_NAME = "AR PL UMing TW"
+DEFAULT_PDF_FONT_FILE = "uming.ttc"
+DEFAULT_PDF_FONT_SIZE = 12
+DEFAULT_PDF_PAGE_SIZE = "A5"
+
 __all__ = [
     "DEFAULT_RE_AUTHOR",
     "DEFAULT_RE_CHAPTER",
+    "DEFAULT_RE_TAG",
     "DEFAULT_RE_TITLE",
+    "DEFAULT_RE_TRANSLATOR",
     "DEFAULT_RE_VOLUME",
     "FULLWIDTH_NUMS",
     "HALFWIDTH_NUMS",
