@@ -1,4 +1,4 @@
-# Copyright (C) 2021,2022,2023 Kian-Meng Ang
+# Copyright (C) 2021,2022,2023,2024 Kian-Meng Ang
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -53,6 +53,7 @@ class Tokenizer:
     """Tokenizer class to parse text content."""
 
     raw_content: str = field(repr=False)
+    metadata_marker: str = field(repr=False)
     config: argparse.Namespace = field(repr=False)
     tokens: List[Token] = field(default_factory=List, repr=False)
     lineno_lookup: Dict = field(default_factory=Dict, repr=False)
@@ -62,6 +63,11 @@ class Tokenizer:
         self.raw_content = raw_content
         self.config = config
 
+        if self.config.fullwidth:
+            self.metadata_marker = "－－－"
+        else:
+            self.metadata_marker = "---"
+
         config_lang = config.language.replace("-", "_")
         self.langconf = import_module(f"txt2ebook.languages.{config_lang}")
 
@@ -71,29 +77,8 @@ class Tokenizer:
             lineno_lookup[line[:10]] = lineno
         self.lineno_lookup = lineno_lookup
 
-        self.tokens = self.parse()
-
-    def __getattr__(self, key: str) -> Any:
-        """Get a value of the config based on key name.
-
-        This function is called when an attribute is not found on an object
-        instance or not set in `__init__` function.
-
-        Args:
-            key(str): The key attribute name of the config, language config,
-            and current class.
-
-        Returns:
-            Any: The value of a key, if found. Otherwise raise AttributeError
-            exception.
-        """
-        if hasattr(self.config, key):
-            return getattr(self.config, key)
-
-        if hasattr(self.langconf, key):
-            return getattr(self.langconf, key)
-
-        raise AttributeError(f"invalid config key: '{key}'!")
+        self.tokens = []
+        self.parse()
 
     def __repr__(self) -> str:
         """Return the string representation of Tokenizer for debugging purpose.
@@ -106,27 +91,14 @@ class Tokenizer:
             self.__class__.__name__, self.raw_content[:5], self.stats()
         )
 
-    def parse(self) -> List:
+    def parse(self) -> None:
         """Parse the content into tokens.
 
         Returns:
-          List: The list of tokens.
+          None
         """
-        content = self.raw_content.rstrip(self.paragraph_separator)
-        lines = content.split(self.paragraph_separator)
-
-        if len(lines) <= 1:
-            msg = (
-                f"Cannot split content by {repr(self.paragraph_separator)}. "
-                "Check if content have newline with spaces."
-            )
-            log_or_raise_on_warning(msg, self.config.raise_on_warning)
-
-        tokens: List[Token] = []
-        for line in lines:
-            self._tokenize_line(line, tokens)
-
-        return tokens
+        self._tokenize_metadata()
+        self._tokenize_content()
 
     def stats(self) -> Counter:
         """Returns the statistics count for the parsed tokens.
@@ -138,68 +110,100 @@ class Tokenizer:
         logger.debug("Token stats: %s", repr(stats))
         return stats
 
-    def _tokenize_line(self, line: str, tokens: List) -> None:
+    def _tokenize_line(self, line: str) -> None:
         """Tokenize each line after we split by paragraph separator."""
-        _ = (
-            self._tokenize_header(line, tokens)
-            or self._tokenize_metadata(line, tokens)
-            or self._tokenize_paragraph(line, tokens)
-        )
+        _ = self._tokenize_header(line) or self._tokenize_paragraph(line)
 
-    def _tokenize_metadata(self, line: str, tokens: List) -> bool:
-        """Tokenize the metadata of the book.
+    def _tokenize_metadata(self) -> None:
+        """Tokenize the metadata of the book."""
+        for line in self._extract_metadata():
+            re_title = f"^{self.langconf.DEFAULT_RE_TITLE}"
+            if self.config.re_title:
+                re_title = self.config.re_title[0]
 
-        Metadata at the top of the file was grouped and separate as single
-        newline. By default, the content, or paragraph was separated by two
-        newlines. Hence, we have to group it into one token.
+            re_author = f"{self.langconf.DEFAULT_RE_AUTHOR}"
+            if self.config.re_author:
+                re_author = self.config.re_author[0]
+
+            token_type_regex_map = [
+                ("TITLE", re_title),
+                ("AUTHOR", re_author),
+                ("TAG", f"{self.langconf.DEFAULT_RE_TAG}"),
+                ("TRANSLATOR", f"{self.langconf.DEFAULT_RE_TRANSLATOR}"),
+            ]
+
+            token = None
+            for token_type, regex in token_type_regex_map:
+                match = re.search(regex, line)
+                if match:
+                    token_value = match.group(1).strip()
+                    token = Token(
+                        token_type, token_value, self._lineno(token_value)
+                    )
+                    self.tokens.append(token)
+
+    def _extract_metadata(self) -> List:
+        """Extract YAML-inspired metadata header from file context.
+
+        Metadata header with line number as follows:
+
+            1 ---
+            2 书名：
+            3 作者：
+            4 标签：
+            5 翻译：
+            6 ---
 
-        Also, we can split the metadata line as these lines can also contains
-        chapter content, which can also contains newlines.
         """
-        re_title = f"^{self.DEFAULT_RE_TITLE}"
-        if self.config.re_title:
-            re_title = self.config.re_title[0]
+        match = re.search(
+            rf"^(?:{self.metadata_marker})\n(.*)\n(?:{self.metadata_marker})$",
+            self.raw_content,
+            re.MULTILINE | re.DOTALL,
+        )
+        if not match:
+            msg = "Missing or invalid metadata."
+            log_or_raise_on_warning(msg, self.config.raise_on_warning)
+            return []
 
-        re_author = f"\n{self.DEFAULT_RE_AUTHOR}"
-        if self.config.re_author:
-            re_author = self.config.re_author[0]
+        metadata = match[1].split("\n")
+        logger.debug("Metadata: %s", metadata)
+        return metadata
 
-        token_type_regex_map = [
-            ("TITLE", re_title),
-            ("AUTHOR", re_author),
-            ("TAG", f"\n{self.DEFAULT_RE_TAG}"),
-        ]
+    def _tokenize_content(self) -> None:
+        content = self.raw_content.split(f"{self.metadata_marker}\n\n")[1]
+        content = content.strip(self.config.paragraph_separator)
+        lines = content.split(self.config.paragraph_separator)
 
-        token = None
-        for token_type, regex in token_type_regex_map:
-            match = re.search(regex, line)
-            if match:
-                token_value = match.group(1).strip()
-                token = Token(
-                    token_type, token_value, self._lineno(token_value)
-                )
-                tokens.append(token)
+        if len(lines) <= 1:
+            msg = (
+                "Cannot split content by "
+                f"{repr(self.config.paragraph_separator)}. "
+                "Check if content have newline with spaces."
+            )
+            log_or_raise_on_warning(msg, self.config.raise_on_warning)
 
-        return bool(token)
+        for line in lines:
+            self._tokenize_line(line)
 
-    def _tokenize_header(self, line: str, tokens: List) -> bool:
+    def _tokenize_header(self, line: str) -> bool:
         """Tokenize section headers.
 
         Note that we parse in such sequence: chapter, volume, volume_chapter to
         prevent unnecessary calls as we've more chapters than volumes.
         """
         return (
-            self._tokenize_chapter(line, tokens)
-            or self._tokenize_volume_chapter(line, tokens)
-            or self._tokenize_volume(line, tokens)
+            self._tokenize_chapter(line)
+            or self._tokenize_volume_chapter(line)
+            or self._tokenize_volume(line)
         )
 
-    def _tokenize_volume_chapter(self, line: str, tokens: List) -> bool:
+    def _tokenize_volume_chapter(self, line: str) -> bool:
         line = self._validate_section_header("volume chapter", line)
         token = None
 
         re_volume_chapter = (
-            rf"^{self.DEFAULT_RE_VOLUME}\s*{self.DEFAULT_RE_CHAPTER}"
+            rf"^{self.langconf.DEFAULT_RE_VOLUME}\s*"
+            rf"{self.langconf.DEFAULT_RE_CHAPTER}"
         )
         if self.config.re_volume_chapter:
             re_volume_chapter = self.config.re_volume_chapter[0]
@@ -215,15 +219,15 @@ class Tokenizer:
                     Token("CHAPTER", chapter, self._lineno(chapter)),
                 ],
             )
-            tokens.append(token)
+            self.tokens.append(token)
 
         return bool(token)
 
-    def _tokenize_volume(self, line: str, tokens: List) -> bool:
+    def _tokenize_volume(self, line: str) -> bool:
         line = self._validate_section_header("volume", line)
         token = None
 
-        re_volume = rf"^{self.DEFAULT_RE_VOLUME}$"
+        re_volume = rf"^{self.langconf.DEFAULT_RE_VOLUME}$"
         if self.config.re_volume:
             re_volume = "(" + "|".join(self.config.re_volume) + ")"
 
@@ -231,15 +235,15 @@ class Tokenizer:
         if match:
             volume = match.group(1).strip()
             token = Token("VOLUME", volume, self._lineno(volume))
-            tokens.append(token)
+            self.tokens.append(token)
 
         return bool(token)
 
-    def _tokenize_chapter(self, line: str, tokens: List) -> bool:
+    def _tokenize_chapter(self, line: str) -> bool:
         line = self._validate_section_header("chapter", line)
         token = None
 
-        re_chapter = rf"^{self.DEFAULT_RE_CHAPTER}$"
+        re_chapter = rf"^{self.langconf.DEFAULT_RE_CHAPTER}$"
         if self.config.re_chapter:
             re_chapter = "(" + "|".join(self.config.re_chapter) + ")"
 
@@ -247,12 +251,12 @@ class Tokenizer:
         if match:
             chapter = match.group(1).strip()
             token = Token("CHAPTER", chapter, self._lineno(chapter))
-            tokens.append(token)
+            self.tokens.append(token)
 
         return bool(token)
 
-    def _tokenize_paragraph(self, line: str, tokens: List) -> bool:
-        tokens.append(Token("PARAGRAPH", line, self._lineno(line)))
+    def _tokenize_paragraph(self, line: str) -> bool:
+        self.tokens.append(Token("PARAGRAPH", line, self._lineno(line)))
         return True
 
     def _validate_section_header(self, header_type: str, line: str) -> str:
