@@ -1,5 +1,5 @@
 # pylint: disable=no-value-for-parameter
-# Copyright (C) 2021,2022,2023 Kian-Meng Ang
+# Copyright (C) 2021,2022,2023,2024 Kian-Meng Ang
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Affero General Public License as published by
@@ -14,7 +14,12 @@
 # You should have received a copy of the GNU Affero General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-"""txt2ebook/tte is a cli tool to convert txt file to ebook format."""
+"""txt2ebook/tte is a cli tool to convert txt file to ebook format.
+
+  website: https://github.com/kianmeng/txt2ebook
+  changelog: https://github.com/kianmeng/txt2ebook/blob/master/CHANGELOG.md
+  issues: https://github.com/kianmeng/txt2ebook/issues
+"""
 
 import argparse
 import logging
@@ -26,7 +31,12 @@ from langdetect import detect
 
 from txt2ebook import __version__, setup_logger
 from txt2ebook.exceptions import EmptyFileError
-from txt2ebook.formats import EBOOK_FORMATS, EPUB_TEMPLATES, create_format
+from txt2ebook.formats import (
+    EBOOK_FORMATS,
+    EPUB_TEMPLATES,
+    PAGE_SIZES,
+    create_format,
+)
 from txt2ebook.parser import Parser
 
 logger = logging.getLogger(__name__)
@@ -82,13 +92,14 @@ def build_parser(
 
     parser = argparse.ArgumentParser(
         add_help=False,
-        description=_doc(),
+        description=__doc__,
         formatter_class=lambda prog: argparse.RawTextHelpFormatter(
             prog, max_help_position=6
         ),
     )
 
     epub = parser.add_argument_group("--format epub")
+    pdf = parser.add_argument_group("--format pdf")
     txt = parser.add_argument_group("--format txt")
     zhlang = parser.add_argument_group("--language zh-cn / --language zh-tw")
 
@@ -109,6 +120,26 @@ def build_parser(
         metavar="EBOOK_FILENAME",
     )
 
+    parser.add_argument(
+        "-of",
+        "--output-folder",
+        dest="output_folder",
+        default="output",
+        help="set default output folder (default: '%(default)s')",
+    )
+
+    parser.add_argument(
+        "-p",
+        "--purge",
+        default=False,
+        action="store_true",
+        dest="purge",
+        help=(
+            "remove converted ebooks specified by --output-folder option "
+            "(default: '%(default)s')"
+        ),
+    )
+
     parser.add_argument(
         "-f",
         "--format",
@@ -146,6 +177,16 @@ def build_parser(
         metavar="AUTHOR",
     )
 
+    parser.add_argument(
+        "-tr",
+        "--translator",
+        dest="translator",
+        default=[],
+        action="append",
+        help="translator of the ebook (default: '%(default)s')",
+        metavar="TRANSLATOR",
+    )
+
     epub.add_argument(
         "-c",
         "--cover",
@@ -165,6 +206,21 @@ def build_parser(
         metavar="WIDTH",
     )
 
+    parser.add_argument(
+        "-ff",
+        "--filename-format",
+        dest="filename_format",
+        type=int,
+        default=None,
+        help=(
+            "the output filename format "
+            "(default: TXT_FILENAME [EBOOK_FILENAME])\n"
+            "1 - title_authors.EBOOK_EXTENSION\n"
+            "2 - authors_title.EBOOK_EXTENSION"
+        ),
+        metavar="FILENAME_FORMAT",
+    )
+
     parser.add_argument(
         "-ps",
         "--paragraph_separator",
@@ -175,13 +231,23 @@ def build_parser(
         metavar="SEPARATOR",
     )
 
+    pdf.add_argument(
+        "-pz",
+        "--page-size",
+        dest="page_size",
+        default=None,
+        choices=PAGE_SIZES,
+        help="page size of the ebook (default: '%(default)s')",
+        metavar="PAGE_SIZE",
+    )
+
     parser.add_argument(
         "-rd",
         "--regex-delete",
         dest="re_delete",
         default=[],
         action="append",
-        help=("regex to delete word or phrase " "(default: '%(default)s')"),
+        help="regex to delete word or phrase (default: '%(default)s')",
         metavar="REGEX",
     )
 
@@ -204,7 +270,7 @@ def build_parser(
         dest="re_volume",
         default=[],
         action="append",
-        help=("regex to parse volume header " "(default: by LANGUAGE)"),
+        help="regex to parse volume header (default: by LANGUAGE)",
         metavar="REGEX",
     )
 
@@ -214,7 +280,7 @@ def build_parser(
         dest="re_chapter",
         default=[],
         action="append",
-        help=("regex to parse chapter header " "(default: by LANGUAGE)"),
+        help="regex to parse chapter header (default: by LANGUAGE)",
         metavar="REGEX",
     )
 
@@ -224,7 +290,7 @@ def build_parser(
         dest="re_title",
         default=[],
         action="append",
-        help=("regex to parse title of the book " "(default: by LANGUAGE)"),
+        help="regex to parse title of the book (default: by LANGUAGE)",
         metavar="REGEX",
     )
 
@@ -234,7 +300,7 @@ def build_parser(
         dest="re_author",
         default=[],
         action="append",
-        help=("regex to parse author of the book " "(default: by LANGUAGE)"),
+        help="regex to parse author of the book (default: by LANGUAGE)",
         metavar="REGEX",
     )
 
@@ -244,7 +310,7 @@ def build_parser(
         dest="re_delete_line",
         default=[],
         action="append",
-        help="regex to delete whole line " "(default: '%(default)s')",
+        help="regex to delete whole line (default: '%(default)s')",
         metavar="REGEX",
     )
 
@@ -255,7 +321,7 @@ def build_parser(
         nargs=2,
         default=[],
         action="append",
-        help="regex to search and replace " "(default: '%(default)s')",
+        help="regex to search and replace (default: '%(default)s')",
         metavar="REGEX",
     )
 
@@ -298,6 +364,15 @@ def build_parser(
         ),
     )
 
+    parser.add_argument(
+        "-ss",
+        "--sort-volume-and-chapter",
+        default=False,
+        action="store_true",
+        dest="sort_volume_and_chapter",
+        help="short volume and chapter",
+    )
+
     txt.add_argument(
         "-toc",
         "--table-of-content",
@@ -343,6 +418,15 @@ def build_parser(
         help="overwrite massaged TXT_FILENAME",
     )
 
+    parser.add_argument(
+        "-q",
+        "--quiet",
+        default=False,
+        action="store_true",
+        dest="quiet",
+        help="suppress all logging",
+    )
+
     parser.add_argument(
         "-v",
         "--verbose",
@@ -352,6 +436,15 @@ def build_parser(
         help="show verbosity of debugging log, use -vv, -vvv for more details",
     )
 
+    parser.add_argument(
+        "-y",
+        "--yes",
+        default=False,
+        action="store_true",
+        dest="yes",
+        help="yes to prompt",
+    )
+
     parser.add_argument(
         "-d",
         "--debug",
@@ -376,14 +469,6 @@ def build_parser(
     return parser
 
 
-def _doc() -> str:
-    return (
-        __doc__
-        + "\n  website: https://github.com/kianmeng/txt2ebook"
-        + "\n  issues: https://github.com/kianmeng/txt2ebook/issues"
-    )
-
-
 def main(args: Optional[Sequence[str]] = None):
     """Set the main entrypoint of the CLI script."""
     args = args or sys.argv[1:]
